@@ -2,6 +2,7 @@ import asyncio
 import enum
 import json
 import logging
+import sys
 from contextlib import suppress
 from dataclasses import dataclass, field
 from datetime import datetime, timezone
@@ -42,7 +43,7 @@ from .images import (
 )
 from .parser import Parser, Volume
 from .parsing_utils import LocalImage, RemoteImage, _as_repo_str, _is_in_neuro_registry
-from .url_utils import normalize_storage_path_uri
+from .url_utils import normalize_secret_uri, normalize_storage_path_uri
 from .utils import NoPublicConstructor, asynccontextmanager
 
 
@@ -87,15 +88,24 @@ class HTTPPort:
     requires_auth: bool = True
 
 
+@dataclass(frozen=True)
+class SecretFile:
+    secret_uri: URL
+    container_path: str
+
+
 @dataclass(frozen=True)
 class Container:
     image: RemoteImage
     resources: Resources
     entrypoint: Optional[str] = None
     command: Optional[str] = None
+    working_dir: Optional[str] = None
     http: Optional[HTTPPort] = None
     env: Mapping[str, str] = field(default_factory=dict)
     volumes: Sequence[Volume] = field(default_factory=list)
+    secret_env: Mapping[str, URL] = field(default_factory=dict)
+    secret_files: Sequence[SecretFile] = field(default_factory=list)
     tty: bool = False
 
 
@@ -138,6 +148,7 @@ class JobDescription:
     http_url: URL = URL()
     ssh_server: URL = URL()
     internal_hostname: Optional[str] = None
+    internal_hostname_named: Optional[str] = None
     restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER
     life_span: Optional[float] = None
 
@@ -388,7 +399,8 @@ class Jobs(metaclass=NoPublicConstructor):
             yield
         finally:
             srv.close()
-            await srv.wait_closed()
+            if sys.version_info >= (3, 7):
+                await srv.wait_closed()
 
     async def _port_forward(
         self,
@@ -420,10 +432,15 @@ class Jobs(metaclass=NoPublicConstructor):
                         with suppress(asyncio.CancelledError):
                             await task
                 writer.close()
-                await writer.wait_closed()
+                if sys.version_info >= (3, 7):
+                    await writer.wait_closed()
                 await ws.close()
         except asyncio.CancelledError:
             raise
+        except WSServerHandshakeError as e:
+            if e.headers and "X-Error" in e.headers:
+                log.error(f"Error during port-forwarding: {e.headers['X-Error']}")
+            log.exception("Unhandled exception during port-forwarding")
         except Exception:
             log.exception("Unhandled exception during port-forwarding")
 
@@ -435,7 +452,8 @@ class Jobs(metaclass=NoPublicConstructor):
             writer.write(msg.data)
             await writer.drain()
         writer.close()
-        await writer.wait_closed()
+        if sys.version_info >= (3, 7):
+            await writer.wait_closed()
 
     async def _port_writer(
         self, ws: aiohttp.ClientWebSocketResponse, reader: asyncio.StreamReader
@@ -551,7 +569,7 @@ class Jobs(metaclass=NoPublicConstructor):
 
 
 def _load_chunk(chunk: bytes) -> Dict[str, Any]:
-    return json.loads(chunk, encoding="utf-8")
+    return json.loads(chunk.decode())
 
 
 def _parse_commit_started_chunk(
@@ -643,9 +661,12 @@ def _container_from_api(data: Dict[str, Any], parse: Parser) -> Container:
         resources=_resources_from_api(data["resources"]),
         entrypoint=data.get("entrypoint", None),
         command=data.get("command", None),
+        working_dir=data.get("working_dir"),
         http=_http_port_from_api(data["http"]) if "http" in data else None,
         env=data.get("env", dict()),
         volumes=[_volume_from_api(v) for v in data.get("volumes", [])],
+        secret_env={name: URL(val) for name, val in data.get("secret_env", {}).items()},
+        secret_files=[_secret_file_from_api(v) for v in data.get("secret_volumes", [])],
         tty=data.get("tty", False),
     )
 
@@ -659,12 +680,23 @@ def _container_to_api(container: Container, config: Config) -> Dict[str, Any]:
         primitive["entrypoint"] = container.entrypoint
     if container.command:
         primitive["command"] = container.command
+    if container.working_dir:
+        primitive["working_dir"] = container.working_dir
     if container.http:
         primitive["http"] = _http_port_to_api(container.http)
     if container.env:
         primitive["env"] = container.env
     if container.volumes:
         primitive["volumes"] = [_volume_to_api(v, config) for v in container.volumes]
+    if container.secret_env:
+        primitive["secret_env"] = {
+            k: str(normalize_secret_uri(v, config.username, config.cluster_name))
+            for k, v in container.secret_env.items()
+        }
+    if container.secret_files:
+        primitive["secret_volumes"] = [
+            _secret_file_to_api(v, config) for v in container.secret_files
+        ]
     if container.tty:
         primitive["tty"] = True
     return primitive
@@ -690,6 +722,7 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
     http_url_named = URL(res.get("http_url_named", ""))
     ssh_server = URL(res.get("ssh_server", ""))
     internal_hostname = res.get("internal_hostname", None)
+    internal_hostname_named = res.get("internal_hostname_named", None)
     restart_policy = JobRestartPolicy(res.get("restart_policy", JobRestartPolicy.NEVER))
     max_run_time_minutes = res.get("max_run_time_minutes")
     life_span = (
@@ -709,6 +742,7 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
         http_url=http_url_named or http_url,
         ssh_server=ssh_server,
         internal_hostname=internal_hostname,
+        internal_hostname_named=internal_hostname_named,
         uri=URL(res["uri"]),
         restart_policy=restart_policy,
         life_span=life_span,
@@ -737,6 +771,16 @@ def _volume_to_api(volume: Volume, config: Config) -> Dict[str, Any]:
     return resp
 
 
+def _secret_file_to_api(secret_file: SecretFile, config: Config) -> Dict[str, Any]:
+    uri = normalize_secret_uri(
+        secret_file.secret_uri, config.username, config.cluster_name
+    )
+    return {
+        "src_secret_uri": str(uri),
+        "dst_path": secret_file.container_path,
+    }
+
+
 def _volume_from_api(data: Dict[str, Any]) -> Volume:
     storage_uri = URL(data["src_storage_uri"])
     container_path = data["dst_path"]
@@ -746,6 +790,12 @@ def _volume_from_api(data: Dict[str, Any]) -> Volume:
     )
 
 
+def _secret_file_from_api(data: Dict[str, Any]) -> SecretFile:
+    secret_uri = URL(data["src_secret_uri"])
+    container_path = data["dst_path"]
+    return SecretFile(secret_uri, container_path)
+
+
 def _parse_datetime(dt: Optional[str]) -> Optional[datetime]:
     if dt is None:
         return None
