@@ -21,7 +21,6 @@ from typing import (
     Dict,
     Iterable,
     Optional,
-    Sequence,
     Tuple,
     Union,
     cast,
@@ -31,18 +30,26 @@ import aiohttp
 import attr
 from yarl import URL
 
+from neuromation.api.utils import QueuedCall, queue_calls
+
 from .abc import (
+    AbstractDeleteProgress,
     AbstractFileProgress,
     AbstractRecursiveFileProgress,
     StorageProgressComplete,
+    StorageProgressDelete,
     StorageProgressEnterDir,
     StorageProgressFail,
     StorageProgressLeaveDir,
     StorageProgressStart,
     StorageProgressStep,
+    _AsyncAbstractDeleteProgress,
+    _AsyncAbstractFileProgress,
+    _AsyncAbstractRecursiveFileProgress,
 )
 from .config import Config
-from .core import ResourceNotFound, _Core
+from .core import _Core
+from .errors import ResourceNotFound
 from .file_filter import FileFilter
 from .url_utils import (
     _extract_path,
@@ -74,6 +81,7 @@ class FileStatus:
     type: FileStatusType
     modification_time: int
     permission: Action
+    uri: URL
 
     def is_file(self) -> bool:
         return self.type == FileStatusType.FILE
@@ -94,10 +102,16 @@ class Storage(metaclass=NoPublicConstructor):
         self._min_time_diff = 0.0
         self._max_time_diff = 0.0
 
-    def _uri_to_path(self, uri: URL) -> str:
-        uri = normalize_storage_path_uri(
+    def _normalize_uri(self, uri: URL) -> URL:
+        return normalize_storage_path_uri(
             uri, self._config.username, self._config.cluster_name
         )
+
+    def _uri_to_path(self, uri: URL, *, normalized: bool = False) -> str:
+        if not normalized:
+            uri = normalize_storage_path_uri(
+                uri, self._config.username, self._config.cluster_name
+            )
         if not uri.host:
             return ""
         if uri.host != self._config.cluster_name:
@@ -134,22 +148,25 @@ class Storage(metaclass=NoPublicConstructor):
         )
 
     async def ls(self, uri: URL) -> AsyncIterator[FileStatus]:
-        url = self._config.storage_url / self._uri_to_path(uri)
+        uri = self._normalize_uri(uri)
+        url = self._config.storage_url / self._uri_to_path(uri, normalized=True)
         url = url.with_query(op="LISTSTATUS")
         headers = {"Accept": "application/x-ndjson"}
 
         request_time = time.time()
         auth = await self._config._api_auth()
+        # NB: the storage server returns file names in FileStatus for LISTSTATUS
+        # but full path for GETFILESTATUS
         async with self._core.request("GET", url, headers=headers, auth=auth) as resp:
             self._set_time_diff(request_time, resp)
             if resp.headers.get("Content-Type", "").startswith("application/x-ndjson"):
                 async for line in resp.content:
                     status = json.loads(line)["FileStatus"]
-                    yield _file_status_from_api(status)
+                    yield _file_status_from_api_ls(uri, status)
             else:
                 res = await resp.json()
                 for status in res["FileStatuses"]["FileStatus"]:
-                    yield _file_status_from_api(status)
+                    yield _file_status_from_api_ls(uri, status)
 
     async def glob(self, uri: URL, *, dironly: bool = False) -> AsyncIterator[URL]:
         if not _has_magic(uri.path):
@@ -241,7 +258,7 @@ class Storage(metaclass=NoPublicConstructor):
         async with self._core.request("PUT", url, auth=auth) as resp:
             resp  # resp.status == 201
 
-    async def create(self, uri: URL, data: AsyncIterator[bytes]) -> None:
+    async def create(self, uri: URL, data: Union[bytes, AsyncIterator[bytes]]) -> None:
         path = self._uri_to_path(uri)
         assert path, "Creation in root is not allowed"
         url = self._config.storage_url / path
@@ -255,15 +272,20 @@ class Storage(metaclass=NoPublicConstructor):
             resp  # resp.status == 201
 
     async def stat(self, uri: URL) -> FileStatus:
-        url = self._config.storage_url / self._uri_to_path(uri)
+        uri = self._normalize_uri(uri)
+        url = self._config.storage_url / self._uri_to_path(uri, normalized=True)
         url = url.with_query(op="GETFILESTATUS")
         auth = await self._config._api_auth()
 
         request_time = time.time()
+        # NB: the storage server returns file names in FileStatus for LISTSTATUS
+        # but full path for GETFILESTATUS
         async with self._core.request("GET", url, auth=auth) as resp:
             self._set_time_diff(request_time, resp)
             res = await resp.json()
-            return _file_status_from_api(res["FileStatus"])
+            return _file_status_from_api_stat(
+                self._config.cluster_name, res["FileStatus"]
+            )
 
     async def open(self, uri: URL) -> AsyncIterator[bytes]:
         url = self._config.storage_url / self._uri_to_path(uri)
@@ -275,8 +297,13 @@ class Storage(metaclass=NoPublicConstructor):
             async for data in resp.content.iter_any():
                 yield data
 
-    async def rm(self, uri: URL, *, recursive: bool = False) -> None:
-        path = self._uri_to_path(uri)
+    async def rm(
+        self,
+        uri: URL,
+        *,
+        recursive: bool = False,
+        progress: Optional[AbstractDeleteProgress] = None,
+    ) -> None:
         # TODO (asvetlov): add a minor protection against deleting everything from root
         # or user volume root, however force operation here should allow user to delete
         # everything.
@@ -288,19 +315,38 @@ class Storage(metaclass=NoPublicConstructor):
         # if final_path == root_data_path or final_path.parent == root_data_path:
         #     raise ValueError("Invalid path value.")
 
-        if not recursive:
-            stats = await self.stat(uri)
-            if stats.type is FileStatusType.DIRECTORY:
-                raise IsADirectoryError(
-                    errno.EISDIR, "Is a directory, use recursive remove", str(uri)
-                )
+        async_progress: _AsyncAbstractDeleteProgress
+        queue, async_progress = queue_calls(progress)
+        await run_progress(
+            queue, self._rm(uri, recursive=recursive, progress=async_progress)
+        )
+
+    async def _rm(
+        self, uri: URL, *, recursive: bool, progress: _AsyncAbstractDeleteProgress
+    ) -> None:
+        path = self._uri_to_path(uri)
 
         url = self._config.storage_url / path
-        url = url.with_query(op="DELETE")
+        url = url.with_query(op="DELETE", recursive="true" if recursive else "false")
         auth = await self._config._api_auth()
+        base_uri = URL.build(scheme="storage", authority=self._config.cluster_name)
 
-        async with self._core.request("DELETE", url, auth=auth) as resp:
-            resp  # resp.status == 204
+        headers = {"Accept": "application/x-ndjson"}
+
+        async with self._core.request(
+            "DELETE", url, headers=headers, auth=auth
+        ) as resp:
+            if resp.headers.get("Content-Type", "").startswith("application/x-ndjson"):
+                async for line in resp.content:
+                    remove_listing = json.loads(line)
+                    await progress.delete(
+                        StorageProgressDelete(
+                            uri=base_uri / remove_listing["path"].lstrip("/"),
+                            is_dir=remove_listing["is_dir"],
+                        )
+                    )
+            else:
+                pass  # Old server versions do not support delete status streaming
 
     async def mv(self, src: URL, dst: URL) -> None:
         url = self._config.storage_url / self._uri_to_path(src)
@@ -313,7 +359,7 @@ class Storage(metaclass=NoPublicConstructor):
     # high-level helpers
 
     async def _iterate_file(
-        self, src: Path, dst: URL, *, progress: "QueuedProgress",
+        self, src: Path, dst: URL, *, progress: _AsyncAbstractFileProgress,
     ) -> AsyncIterator[bytes]:
         loop = asyncio.get_event_loop()
         src_url = URL(src.as_uri())
@@ -384,11 +430,12 @@ class Storage(metaclass=NoPublicConstructor):
                     ):
                         return
 
-        queued = QueuedProgress(progress)
-        await run_progress(queued, self._upload_file(path, dst, progress=queued))
+        async_progress: _AsyncAbstractFileProgress
+        queue, async_progress = queue_calls(progress)
+        await run_progress(queue, self._upload_file(path, dst, progress=async_progress))
 
     async def _upload_file(
-        self, src_path: Path, dst: URL, *, progress: "QueuedProgress",
+        self, src_path: Path, dst: URL, *, progress: _AsyncAbstractFileProgress,
     ) -> None:
         for retry in retries(f"Fail to upload {dst}"):
             async with retry:
@@ -403,7 +450,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool = False,
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
-        ignore_file_names: Union[Sequence[str], AbstractSet[str]] = (),
+        ignore_file_names: AbstractSet[str] = frozenset(),
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
         if filter is None:
@@ -417,9 +464,10 @@ class Storage(metaclass=NoPublicConstructor):
             raise FileNotFoundError(errno.ENOENT, "No such file", str(path))
         if not path.is_dir():
             raise NotADirectoryError(errno.ENOTDIR, "Not a directory", str(path))
-        queued = QueuedProgress(progress)
+        async_progress: _AsyncAbstractRecursiveFileProgress
+        queue, async_progress = queue_calls(progress)
         await run_progress(
-            queued,
+            queue,
             self._upload_dir(
                 src,
                 path,
@@ -428,7 +476,7 @@ class Storage(metaclass=NoPublicConstructor):
                 update=update,
                 filter=filter,
                 ignore_file_names=ignore_file_names,
-                progress=queued,
+                progress=async_progress,
             ),
         )
 
@@ -441,8 +489,8 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         filter: Callable[[str], Awaitable[bool]],
-        ignore_file_names: Union[Sequence[str], AbstractSet[str]],
-        progress: "QueuedProgress",
+        ignore_file_names: AbstractSet[str],
+        progress: _AsyncAbstractRecursiveFileProgress,
     ) -> None:
         tasks = []
         try:
@@ -550,9 +598,11 @@ class Storage(metaclass=NoPublicConstructor):
                     dst_stat, src_stat
                 ):
                     return
-        queued = QueuedProgress(progress)
+        async_progress: _AsyncAbstractFileProgress
+        queue, async_progress = queue_calls(progress)
         await run_progress(
-            queued, self._download_file(src, dst, path, src_stat.size, progress=queued),
+            queue,
+            self._download_file(src, dst, path, src_stat.size, progress=async_progress),
         )
 
     async def _download_file(
@@ -562,7 +612,7 @@ class Storage(metaclass=NoPublicConstructor):
         dst_path: Path,
         size: int,
         *,
-        progress: "QueuedProgress",
+        progress: _AsyncAbstractFileProgress,
     ) -> None:
         loop = asyncio.get_event_loop()
         async with self._file_sem:
@@ -595,11 +645,19 @@ class Storage(metaclass=NoPublicConstructor):
         )
         dst = normalize_local_path_uri(dst)
         path = _extract_path(dst)
-        queued = QueuedProgress(progress)
+
+        async_progress: _AsyncAbstractRecursiveFileProgress
+        queue, async_progress = queue_calls(progress)
         await run_progress(
-            queued,
+            queue,
             self._download_dir(
-                src, dst, path, "", update=update, filter=filter, progress=queued
+                src,
+                dst,
+                path,
+                "",
+                update=update,
+                filter=filter,
+                progress=async_progress,
             ),
         )
 
@@ -612,7 +670,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         filter: Callable[[str], Awaitable[bool]],
-        progress: "QueuedProgress",
+        progress: _AsyncAbstractRecursiveFileProgress,
     ) -> None:
         dst_path.mkdir(parents=True, exist_ok=True)
         await progress.enter(StorageProgressEnterDir(src, dst))
@@ -694,71 +752,41 @@ def _isrecursive(pattern: str) -> bool:
     return pattern == "**"
 
 
-def _file_status_from_api(values: Dict[str, Any]) -> FileStatus:
+def _file_status_from_api_ls(base_uri: URL, values: Dict[str, Any]) -> FileStatus:
     return FileStatus(
         path=values["path"],
         type=FileStatusType(values["type"]),
         size=int(values["length"]),
         modification_time=int(values["modificationTime"]),
         permission=Action(values["permission"]),
+        uri=base_uri / values["path"],
+    )
+
+
+def _file_status_from_api_stat(cluster_name: str, values: Dict[str, Any]) -> FileStatus:
+    base_uri = URL.build(scheme="storage", authority=cluster_name)
+    return FileStatus(
+        path=values["path"],
+        type=FileStatusType(values["type"]),
+        size=int(values["length"]),
+        modification_time=int(values["modificationTime"]),
+        permission=Action(values["permission"]),
+        uri=base_uri / values["path"].lstrip("/"),
     )
 
 
 ProgressQueueItem = Optional[Tuple[Callable[[Any], None], Any]]
 
 
-class QueuedProgress:
-    def __init__(
-        self, progress: Optional[AbstractFileProgress],
-    ):
-        self._progress = progress
-        self._queue: "asyncio.Queue[ProgressQueueItem]" = asyncio.Queue()
-
-    @property
-    def queue(self) -> "asyncio.Queue[ProgressQueueItem]":
-        return self._queue
-
-    async def start(self, data: StorageProgressStart) -> None:
-        if self._progress is not None:
-            await self._queue.put((self._progress.start, data))
-
-    async def complete(self, data: StorageProgressComplete) -> None:
-        if self._progress is not None:
-            await self._queue.put((self._progress.complete, data))
-
-    async def step(self, data: StorageProgressStep) -> None:
-        if self._progress is not None:
-            await self._queue.put((self._progress.step, data))
-
-    async def enter(self, data: StorageProgressEnterDir) -> None:
-        if self._progress is not None:
-            progress = cast(AbstractRecursiveFileProgress, self._progress)
-            await self._queue.put((progress.enter, data))
-
-    async def leave(self, data: StorageProgressLeaveDir) -> None:
-        if self._progress is not None:
-            progress = cast(AbstractRecursiveFileProgress, self._progress)
-            await self._queue.put((progress.leave, data))
-
-    async def fail(self, data: StorageProgressFail) -> None:
-        if self._progress is not None:
-            progress = cast(AbstractRecursiveFileProgress, self._progress)
-            await self._queue.put((progress.fail, data))
-
-
 async def run_progress(
-    progress: Optional[QueuedProgress], coro: Awaitable[None]
+    queue: "asyncio.Queue[QueuedCall]", coro: Awaitable[None]
 ) -> None:
-    if progress is None:
-        return await coro
-
-    queue = progress.queue
-
     async def wrapped() -> None:
         try:
             await coro
         finally:
-            await queue.put(None)
+            # Add special marker to queue to allow loop below to exit
+            await queue.put(cast(QueuedCall, None))
 
     loop = asyncio.get_event_loop()
     task = loop.create_task(wrapped())
@@ -766,8 +794,7 @@ async def run_progress(
         item = await queue.get()
         if item is None:
             break
-        cb, data = item
-        cb(data)
+        item()
     await task
 
 
