@@ -1,6 +1,8 @@
 # Distributed tracing support
 
 import os
+import random
+import time
 import types
 
 import aiohttp
@@ -8,6 +10,21 @@ from multidict import CIMultiDict
 
 
 def gen_trace_id() -> str:
+    """Return 32 random hexadecimal digits.
+
+    The upper 32 bits are the current time in epoch seconds, and the
+    lower 96 bits are random. This allows for AWS X-Ray `interop
+    <https://github.com/openzipkin/zipkin/issues/1754>`_
+
+    The id is used for distributed tracing.
+    """
+
+    high = int(time.time())
+    low = random.getrandbits(96)
+    return "{:032x}".format((high << 96) | low)
+
+
+def _gen_span_id() -> str:
     """Return 16 random hexadecimal digits.
 
     The id is used for distributed tracing.
@@ -31,7 +48,7 @@ async def _on_request_start(
     trace_id = getattr(trace_ctx, "trace_id", None)
     if trace_id is None:
         return
-    span_id = gen_trace_id()
+    span_id = _gen_span_id()
     _update_headers(params.headers, trace_id, span_id)
 
 
