@@ -73,6 +73,14 @@ async def add_quota(root: Root) -> None:
     )
 
 
+def _print_welcome(url: URL) -> None:
+    click.echo(f"Logged into {url}")
+    click.echo(
+        "Read the docs at https://docs.neu.ro or run `neuro --help` "
+        "to see the reference"
+    )
+
+
 @command(init_client=False)
 @argument("url", required=False, default=DEFAULT_API_URL, type=URL)
 async def login(root: Root, url: URL) -> None:
@@ -92,7 +100,7 @@ async def login(root: Root, url: URL) -> None:
         await root.factory.logout()
         click.echo("You were successfully logged out.")
         await root.factory.login(show_browser, url=url, timeout=root.timeout)
-    click.echo(f"Logged into {url}")
+    _print_welcome(url)
 
 
 @command(init_client=False)
@@ -111,7 +119,7 @@ async def login_with_token(root: Root, token: str, url: URL) -> None:
         await root.factory.logout()
         click.echo("You were successfully logged out.")
         await root.factory.login_with_token(token, url=url, timeout=root.timeout)
-    click.echo(f"Logged into {url}")
+    _print_welcome(url)
 
 
 @command(init_client=False)
@@ -131,7 +139,7 @@ async def login_headless(root: Root, url: URL) -> None:
     """
 
     async def login_callback(url: URL) -> str:
-        session = PromptSession()
+        session: PromptSession[str] = PromptSession()
         click.echo(f"Open {url} in a browser")
         click.echo("Put the code displayed in a browser after successful login")
         with patch_stdout():
@@ -146,7 +154,7 @@ async def login_headless(root: Root, url: URL) -> None:
         await root.factory.logout()
         click.echo("You were successfully logged out.")
         await root.factory.login_headless(login_callback, url=url, timeout=root.timeout)
-    click.echo(f"Logged into {url}")
+    _print_welcome(url)
 
 
 @command(init_client=False)
@@ -158,7 +166,7 @@ async def logout(root: Root) -> None:
     click.echo("Logged out")
 
 
-@command()
+@command(init_client=False)
 async def aliases(root: Root) -> None:
     """
     List available command aliases.
@@ -248,7 +256,7 @@ async def switch_cluster(root: Root, cluster_name: Optional[str]) -> None:
 
 
 async def prompt_cluster(
-    client: Client, *, session: Optional[PromptSession] = None
+    client: Client, *, session: Optional[PromptSession[str]] = None
 ) -> str:
     if session is None:
         session = PromptSession()
