@@ -25,6 +25,7 @@ from neuromation.api import (
     JobStatus,
     RemoteImage,
     Resources,
+    SecretFile,
     Volume,
 )
 from neuromation.cli.formatters.images import DockerImageProgress
@@ -72,6 +73,7 @@ from .utils import (
     group,
     option,
     pager_maybe,
+    parse_secret_resource,
     resolve_job,
     volume_to_verbose_str,
 )
@@ -87,7 +89,7 @@ NEUROMATION_HOME_ENV_VAR = "NEUROMATION_HOME"
 RESERVED_ENV_VARS = {NEUROMATION_ROOT_ENV_VAR, NEUROMATION_HOME_ENV_VAR}
 
 DEFAULT_JOB_LIFE_SPAN = "1d"
-REGEX_JOB_LIFE_SPAN = re.compile(
+REGEX_TIME_DELTA = re.compile(
     r"^((?P<d>\d+)d)?((?P<h>\d+)h)?((?P<m>\d+)m)?((?P<s>\d+)s)?$"
 )
 
@@ -138,6 +140,15 @@ def build_env(env: Sequence[str], env_file: Optional[str]) -> Dict[str, str]:
     return env_dict
 
 
+def _extract_secret_env(env_dict: Dict[str, str], root: Root) -> Dict[str, URL]:
+    secret_env_dict = {}
+    for name, val in env_dict.copy().items():
+        if val.startswith("secret:"):
+            secret_env_dict[name] = parse_secret_resource(val, root)
+            del env_dict[name]
+    return secret_env_dict
+
+
 @group()
 def job() -> None:
     """
@@ -145,7 +156,9 @@ def job() -> None:
     """
 
 
-@command(context_settings=dict(allow_interspersed_args=False))
+@command(
+    context_settings=dict(allow_interspersed_args=False), hidden=True, deprecated=True
+)
 @argument("image", type=ImageType())
 @argument("cmd", nargs=-1, type=click.UNPROCESSED)
 @option(
@@ -245,8 +258,7 @@ def job() -> None:
     help=(
         "Mounts directory from vault into container. "
         "Use multiple options to mount more than one volume. "
-        "--volume=HOME is an alias for storage::/var/storage/home:rw and "
-        "storage://neuromation/public:/var/storage/neuromation:ro"
+        "Use --volume=ALL to mount all accessible storage directories."
     ),
     secure=True,
 )
@@ -306,6 +318,14 @@ def job() -> None:
     show_default=True,
     help="Upload neuro config to the job",
 )
+@option(
+    "--port-forward",
+    type=LOCAL_REMOTE_PORT,
+    multiple=True,
+    metavar="LOCAL_PORT:REMOTE_RORT",
+    help="Forward port(s) of a running job to local port(s) "
+    "(use multiple times for forwarding several ports)",
+)
 @option("--browse", is_flag=True, help="Open a job's URL in a web browser")
 @option(
     "--detach",
@@ -327,11 +347,13 @@ async def submit(
     http_auth: Optional[bool],
     entrypoint: Optional[str],
     cmd: Sequence[str],
+    working_dir: Optional[str],
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
     restart: str,
     life_span: Optional[str],
+    port_forward: List[Tuple[int, int]],
     preemptible: bool,
     name: Optional[str],
     tag: Sequence[str],
@@ -377,11 +399,13 @@ async def submit(
         http_auth=http_auth,
         entrypoint=entrypoint,
         cmd=cmd,
+        working_dir=working_dir,
         volume=volume,
         env=env,
         env_file=env_file,
         restart=restart,
         life_span=life_span,
+        port_forward=port_forward,
         preemptible=preemptible,
         name=name,
         tags=tag,
@@ -391,6 +415,7 @@ async def submit(
         browse=browse,
         detach=detach,
         tty=tty,
+        schedule_timeout=None,
     )
 
 
@@ -443,11 +468,18 @@ async def exec(
 
 @command()
 @argument("job", type=JOB)
-@argument("local_remote_port", type=LOCAL_REMOTE_PORT, nargs=-1, required=True)
+@argument(
+    "local_remote_port",
+    type=LOCAL_REMOTE_PORT,
+    nargs=-1,
+    required=True,
+    metavar="LOCAL_PORT:REMOTE_RORT...",
+)
 @option(
     "--no-key-check",
     is_flag=True,
     help="Disable host key checks. Should be used with caution.",
+    hidden=True,
 )
 async def port_forward(
     root: Root, job: str, no_key_check: bool, local_remote_port: List[Tuple[int, int]]
@@ -471,6 +503,13 @@ async def port_forward(
     neuro job port-forward my-job 2080:80 2222:22 2000:100
 
     """
+    if no_key_check:
+        click.secho(
+            "--no-key-check option is deprecated and "
+            "will be removed from the future Neuro CLI release",
+            fg="red",
+            err=True,
+        )
     job_id = await resolve_job(
         job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
     )
@@ -481,9 +520,7 @@ async def port_forward(
                 f"to port {job_port} of {job_id}"
             )
             await stack.enter_async_context(
-                root.client.jobs.port_forward(
-                    job_id, local_port, job_port, no_key_check=no_key_check
-                )
+                root.client.jobs.port_forward(job_id, local_port, job_port)
             )
 
         click.echo("Press ^C to stop forwarding")
@@ -515,7 +552,15 @@ async def logs(root: Root, job: str) -> None:
 
 @command()
 @argument("job", type=JOB)
-async def attach(root: Root, job: str) -> None:
+@option(
+    "--port-forward",
+    type=LOCAL_REMOTE_PORT,
+    multiple=True,
+    metavar="LOCAL_PORT:REMOTE_RORT",
+    help="Forward port(s) of a running job to local port(s) "
+    "(use multiple times for forwarding several ports)",
+)
+async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> None:
     """
     Attach local standard input, output, and error streams to a running job.
     """
@@ -538,7 +583,7 @@ async def attach(root: Root, job: str) -> None:
     tty = status.container.tty
     _check_tty(root, tty)
 
-    await process_attach(root, status, tty=tty, logs=False)
+    await process_attach(root, status, tty=tty, logs=False, port_forward=port_forward)
 
 
 @command()
@@ -702,7 +747,11 @@ async def status(root: Root, job: str, full_uri: bool) -> None:
         uri_fmtr = uri_formatter(
             username=root.client.username, cluster_name=root.client.cluster_name
         )
-    click.echo(JobStatusFormatter(uri_formatter=uri_fmtr)(res))
+    if root.tty:
+        width = root.terminal_size[0]
+    else:
+        width = 0
+    click.echo(JobStatusFormatter(uri_formatter=uri_fmtr, width=width)(res))
 
 
 @command()
@@ -885,8 +934,9 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     help=(
         "Mounts directory from vault into container. "
         "Use multiple options to mount more than one volume. "
-        "--volume=HOME is an alias for storage::/var/storage/home:rw and "
-        "storage://neuromation/public:/var/storage/neuromation:ro"
+        "Use --volume=ALL to mount all accessible storage directories. "
+        "See `neuro help secrets` for information about "
+        "passing secrets as mounted files."
     ),
     secure=True,
 )
@@ -900,14 +950,23 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     ),
     secure=True,
 )
+@option(
+    "-w",
+    "--workdir",
+    type=str,
+    help="Working directory inside the container",
+    secure=True,
+)
 @option(
     "-e",
     "--env",
     metavar="VAR=VAL",
     multiple=True,
     help=(
-        "Set environment variable in container "
-        "Use multiple options to define more than one variable"
+        "Set environment variable in container. "
+        "Use multiple options to define more than one variable. "
+        "See `neuro help secrets` for information about "
+        "passing secrets as environment variables."
     ),
     secure=True,
 )
@@ -935,6 +994,16 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     ),
     show_default=True,
 )
+@option(
+    "--schedule-timeout",
+    type=str,
+    metavar="TIMEDELTA",
+    help=(
+        "Optional job schedule timeout in the format '3m4s' "
+        "(some parts may be missing)."
+    ),
+    show_default=True,
+)
 @option(
     "--wait-start/--no-wait-start",
     default=True,
@@ -947,6 +1016,14 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     show_default=True,
     help="Upload neuro config to the job",
 )
+@option(
+    "--port-forward",
+    type=LOCAL_REMOTE_PORT,
+    multiple=True,
+    metavar="LOCAL_PORT:REMOTE_RORT",
+    help="Forward port(s) of a running job to local port(s) "
+    "(use multiple times for forwarding several ports)",
+)
 @option("--browse", is_flag=True, help="Open a job's URL in a web browser")
 @option(
     "--detach",
@@ -963,6 +1040,7 @@ async def run(
     http_auth: Optional[bool],
     entrypoint: Optional[str],
     cmd: Sequence[str],
+    workdir: Optional[str],
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
@@ -974,9 +1052,11 @@ async def run(
     description: Optional[str],
     wait_start: bool,
     pass_config: bool,
+    port_forward: List[Tuple[int, int]],
     browse: bool,
     detach: bool,
     tty: Optional[bool],
+    schedule_timeout: Optional[str],
 ) -> None:
     """
     Run a job with predefined resources configuration.
@@ -989,9 +1069,10 @@ async def run(
 
     # Starts a container pytorch:latest on a machine with smaller GPU resources
     # (see exact values in `neuro config show`) and with two volumes mounted:
-    #   storage://<home-directory>   --> /var/storage/home (in read-write mode),
-    #   storage://neuromation/public --> /var/storage/neuromation (in read-only mode).
-    neuro run --preset=gpu-small --volume=HOME pytorch:latest
+    #   storage:/<home-directory>   --> /var/storage/home (in read-write mode),
+    #   storage:/neuromation/public --> /var/storage/neuromation (in read-only mode).
+    neuro run --preset=gpu-small --volume=storage::/var/storage/home:rw \\\\
+        --volume=storage:/neuromation/public:/var/storage/home:ro pytorch:latest
 
     # Starts a container using the custom image my-ubuntu:latest stored in neuromation
     # registry, run /script.sh and pass arg1 and arg2 as its arguments:
@@ -1021,11 +1102,13 @@ async def run(
         http_auth=http_auth,
         entrypoint=entrypoint,
         cmd=cmd,
+        working_dir=workdir,
         volume=volume,
         env=env,
         env_file=env_file,
         restart=restart,
         life_span=life_span,
+        port_forward=port_forward,
         preemptible=job_preset.is_preemptible,
         name=name,
         tags=tag,
@@ -1035,6 +1118,7 @@ async def run(
         browse=browse,
         detach=detach,
         tty=tty,
+        schedule_timeout=schedule_timeout,
     )
 
 
@@ -1072,11 +1156,13 @@ async def run_job(
     http_auth: Optional[bool],
     entrypoint: Optional[str],
     cmd: Sequence[str],
+    working_dir: Optional[str],
     volume: Sequence[str],
     env: Sequence[str],
     env_file: Optional[str],
     restart: str,
     life_span: Optional[str],
+    port_forward: List[Tuple[int, int]],
     preemptible: bool,
     name: Optional[str],
     tags: Sequence[str],
@@ -1086,6 +1172,7 @@ async def run_job(
     browse: bool,
     detach: bool,
     tty: bool,
+    schedule_timeout: Optional[str],
 ) -> JobDescription:
     if http_auth is None:
         http_auth = True
@@ -1109,7 +1196,14 @@ async def run_job(
     job_life_span = await calc_life_span(root.client, life_span)
     log.debug(f"Job run-time limit: {job_life_span}")
 
+    if schedule_timeout is None:
+        job_schedule_timeout = None
+    else:
+        job_schedule_timeout = _parse_timedelta(schedule_timeout).total_seconds()
+    log.debug(f"Job schedule timeout: {job_schedule_timeout}")
+
     env_dict = build_env(env, env_file)
+    secret_env_dict = _extract_secret_env(env_dict, root)
     real_cmd = _parse_cmd(cmd)
 
     log.debug(f'entrypoint="{entrypoint}"')
@@ -1131,7 +1225,10 @@ async def run_job(
         tpu_type=tpu_type,
         tpu_software_version=tpu_software_version,
     )
-    volumes = await _build_volumes(root, volume, env_dict)
+    input_secret_files = {vol for vol in volume if vol.startswith("secret:")}
+    input_volumes = set(volume) - input_secret_files
+    secret_files = await _build_secret_files(root, input_secret_files)
+    volumes = await _build_volumes(root, input_volumes, env_dict)
 
     if pass_config:
         env_name = NEURO_STEAL_CONFIG
@@ -1151,10 +1248,13 @@ async def run_job(
         image=image,
         entrypoint=entrypoint,
         command=real_cmd,
+        working_dir=working_dir,
         http=HTTPPort(http, http_auth) if http else None,
         resources=resources,
         env=env_dict,
         volumes=list(volumes),
+        secret_env=secret_env_dict,
+        secret_files=list(secret_files),
         tty=tty,
     )
 
@@ -1166,6 +1266,7 @@ async def run_job(
         description=description,
         restart_policy=job_restart_policy,
         life_span=job_life_span,
+        schedule_timeout=job_schedule_timeout,
     )
     progress = JobStartProgress.create(tty=root.tty, color=root.color, quiet=root.quiet)
     progress.begin(job)
@@ -1183,7 +1284,7 @@ async def run_job(
         await browse_job(root, job)
 
     if not detach:
-        await process_attach(root, job, tty=tty, logs=True)
+        await process_attach(root, job, tty=tty, logs=True, port_forward=port_forward)
 
     return job
 
@@ -1197,14 +1298,13 @@ def _parse_cmd(cmd: Sequence[str]) -> str:
 
 
 async def _build_volumes(
-    root: Root, input_volumes: Sequence[str], env_dict: Dict[str, str]
+    root: Root, input_volumes: Set[str], env_dict: Dict[str, str]
 ) -> Set[Volume]:
     cluster_name = root.client.cluster_name
-    input_volumes_set = set(input_volumes)
     volumes: Set[Volume] = set()
 
-    if "ALL" in input_volumes_set:
-        if len(input_volumes_set) > 1:
+    if "ALL" in input_volumes:
+        if len(input_volumes) > 1:
             raise click.UsageError(
                 f"Cannot use `--volume=ALL` together with other `--volume` options"
             )
@@ -1232,31 +1332,25 @@ async def _build_volumes(
                 f"  {NEUROMATION_HOME_ENV_VAR}={neuro_mountpoint}"
             )
     else:
-        for vol in input_volumes_set:
-            if vol == "HOME":
-                volumes.add(
-                    root.client.parse.volume(f"storage::{STORAGE_MOUNTPOINT}/home:rw")
-                )
-                volumes.add(
-                    root.client.parse.volume(
-                        f"storage://{cluster_name}/neuromation/public:"
-                        f"{STORAGE_MOUNTPOINT}/neuromation:ro"
-                    )
-                )
-                click.echo(
-                    click.style(
-                        "DeprecationWarning: Option `--volume=HOME` is deprecated. "
-                        "Use `--volume=ALL`.  Mountpoint will be available in "
-                        "container via variable NEUROMATION_HOME",
-                        fg="red",
-                    ),
-                    err=True,
-                )
-            else:
-                volumes.add(root.client.parse.volume(vol))
+        if "HOME" in input_volumes:
+            raise ValueError("--volume=HOME no longer supported")
+        for vol in input_volumes:
+            volumes.add(root.client.parse.volume(vol))
     return volumes
 
 
+async def _build_secret_files(root: Root, input_volumes: Set[str]) -> Set[SecretFile]:
+    secret_files: Set[SecretFile] = set()
+    for volume in input_volumes:
+        parts = volume.split(":")
+        if len(parts) != 3:
+            raise ValueError(f"Invalid secret file specification '{volume}'")
+        container_path = parts.pop()
+        secret_uri = parse_secret_resource(":".join(parts), root)
+        secret_files.add(SecretFile(secret_uri, container_path))
+    return secret_files
+
+
 async def upload_and_map_config(root: Root) -> Tuple[str, Volume]:
 
     # store the Neuro CLI config on the storage under some random path
@@ -1368,12 +1462,12 @@ async def calc_life_span(client: Client, value: Optional[str]) -> Optional[float
 
 def _parse_timedelta(value: str) -> timedelta:
     value = value.strip()
-    err = f"Could not parse job timeout '{value}'"
+    err = f"Could not parse time delta '{value}'"
     if value == "":
         raise click.UsageError(f"{err}: Empty string not allowed")
     if value == "0":
         return timedelta(0)
-    match = REGEX_JOB_LIFE_SPAN.search(value)
+    match = REGEX_TIME_DELTA.search(value)
     if match is None:
         raise click.UsageError(
             f"{err}: Should be like '1d2h3m4s' (some parts may be missing)."
