@@ -145,6 +145,20 @@ def format_example(example: str, formatter: click.HelpFormatter) -> None:
 
 
 class NeuroClickMixin:
+    def get_params(self, ctx: click.Context) -> List[click.Parameter]:
+        # super() is available after using as a mixin
+        ret = super().get_params(ctx)  # type: ignore
+        args = [i for i in ret if not isinstance(i, click.Option)]
+        opts = [i for i in ret if isinstance(i, click.Option)]
+
+        help_names = self.get_help_option_names(ctx)  # type: ignore
+
+        def sort_key(opt: click.Option) -> Tuple[bool, str]:
+            flag = set(opt.opts) & help_names or set(opt.secondary_opts) & help_names
+            return (not flag, opt.name)
+
+        return args + sorted(opts, key=sort_key)
+
     def get_help_option(self, ctx: click.Context) -> Optional[click.Option]:
         help_options = self.get_help_option_names(ctx)  # type: ignore
         if not help_options or not self.add_help_option:  # type: ignore
@@ -436,7 +450,6 @@ async def resolve_job(
     if re.fullmatch(JOB_ID_PATTERN, id_or_name):
         return id_or_name
 
-    details = f"name={id_or_name}, owner={owner}"
     try:
         async for job in client.jobs.list(
             name=id_or_name, owners={owner}, reverse=True, limit=1
@@ -448,9 +461,11 @@ async def resolve_job(
     except Exception as e:
         log.error(
             f"Failed to resolve job-name {id_or_name_or_uri} resolved as "
-            f"{details} to a job-ID: {e}"
+            f"name={id_or_name}, owner={owner} to a job-ID: {e}"
         )
 
+    if owner != default_user:
+        raise ValueError(f"Failed to resolve job {id_or_name_or_uri}")
     return id_or_name
 
 
@@ -506,6 +521,12 @@ def parse_blob_or_file_resource(uri: str, root: Root) -> URL:
     )
 
 
+def parse_secret_resource(uri: str, root: Root) -> URL:
+    return uri_from_cli(
+        uri, root.client.username, root.client.cluster_name, allowed_schemes=("secret"),
+    )
+
+
 def parse_permission_action(action: str) -> Action:
     try:
         return Action[action.upper()]
@@ -567,6 +588,9 @@ def pager_maybe(
             click.echo(line)
         return
 
+    # Enforce ANSI sequence handling (colors etc.)
+    os.environ["LESS"] = "-R"
+
     lines_it: Iterator[str] = iter(lines)
     count = int(terminal_size[1] * 2 / 3)
     handled = list(itertools.islice(lines_it, count))
