@@ -1,3 +1,4 @@
+import os
 import sys
 from pathlib import Path
 from typing import Any, Callable, Dict
@@ -13,14 +14,17 @@ import neuromation
 import neuromation.api.config_factory
 from neuromation.api import Cluster, ConfigError, Factory
 from neuromation.api.config import _AuthConfig, _AuthToken, _ConfigData
-from neuromation.api.login import AuthException
+from neuromation.api.errors import AuthException
 from tests import _TestServerFactory
 
 
 @pytest.fixture
 def tmp_home(tmp_path: Path, monkeypatch: Any) -> Path:
     monkeypatch.setattr(Path, "home", lambda: tmp_path)  # Like as it's not enough
-    monkeypatch.setenv("HOME", str(tmp_path))
+    if os.name == "nt" and sys.version_info >= (3, 8):
+        monkeypatch.setenv("USERPROFILE", str(tmp_path))
+    else:
+        monkeypatch.setenv("HOME", str(tmp_path))
 
     return tmp_path
 
@@ -87,6 +91,7 @@ async def mock_for_login(
                             "storage_url": "https://storage-dev.test.com",
                             "users_url": "https://users-dev.test.com",
                             "monitoring_url": "https://monitoring-dev.test.com",
+                            "secrets_url": "https://secrets-dev.test.com",
                             "resource_presets": [
                                 {
                                     "name": "gpu-small",
