@@ -23,6 +23,7 @@ from neuromation.api import (
     StorageProgressStart,
     StorageProgressStep,
 )
+from neuromation.api.abc import StorageProgressDelete
 from tests import _RawTestServerFactory, _TestServerFactory
 
 
@@ -166,6 +167,7 @@ async def test_storage_ls_legacy(
             type=FileStatusType.FILE,
             modification_time=0,
             permission=Action.READ,
+            uri=URL("storage://default/user/folder/foo"),
         ),
         FileStatus(
             path="bar",
@@ -173,6 +175,7 @@ async def test_storage_ls_legacy(
             type=FileStatusType.DIRECTORY,
             modification_time=0,
             permission=Action.READ,
+            uri=URL("storage://default/user/folder/bar"),
         ),
     ]
 
@@ -231,6 +234,7 @@ async def test_storage_ls(
             type=FileStatusType.FILE,
             modification_time=0,
             permission=Action.READ,
+            uri=URL("storage://default/user/folder/foo"),
         ),
         FileStatus(
             path="bar",
@@ -238,6 +242,7 @@ async def test_storage_ls(
             type=FileStatusType.DIRECTORY,
             modification_time=0,
             permission=Action.READ,
+            uri=URL("storage://default/user/folder/bar"),
         ),
     ]
 
@@ -399,28 +404,19 @@ async def test_storage_glob(
 async def test_storage_rm_file(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
-    async def get_handler(request: web.Request) -> web.Response:
-        assert request.path == "/storage/user/file"
-        assert request.query == {"op": "GETFILESTATUS"}
-        return web.json_response(
-            {
-                "FileStatus": {
-                    "path": "/user/file",
-                    "type": "FILE",
-                    "length": 1234,
-                    "modificationTime": 3456,
-                    "permission": "read",
-                }
-            }
-        )
+    remove_listing = {"path": "/user/file", "is_dir": False}
 
-    async def delete_handler(request: web.Request) -> web.Response:
+    async def delete_handler(request: web.Request) -> web.StreamResponse:
         assert request.path == "/storage/user/file"
-        assert request.query == {"op": "DELETE"}
-        return web.Response(status=204)
+        assert request.query == {"op": "DELETE", "recursive": "false"}
+        assert request.headers["Accept"] == "application/x-ndjson"
+        resp = web.StreamResponse()
+        resp.headers["Content-Type"] = "application/x-ndjson"
+        await resp.prepare(request)
+        await resp.write(json.dumps(remove_listing).encode() + b"\n")
+        return resp
 
     app = web.Application()
-    app.router.add_get("/storage/user/file", get_handler)
     app.router.add_delete("/storage/user/file", delete_handler)
 
     srv = await aiohttp_server(app)
@@ -429,37 +425,53 @@ async def test_storage_rm_file(
         await client.storage.rm(URL("storage:file"))
 
 
+async def test_storage_rm_file_progress(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    remove_listing = {"path": "/user/file", "is_dir": False}
+
+    async def delete_handler(request: web.Request) -> web.StreamResponse:
+        assert request.path == "/storage/user/file"
+        assert request.query == {"op": "DELETE", "recursive": "false"}
+        assert request.headers["Accept"] == "application/x-ndjson"
+        resp = web.StreamResponse()
+        resp.headers["Content-Type"] = "application/x-ndjson"
+        await resp.prepare(request)
+        await resp.write(json.dumps(remove_listing).encode() + b"\n")
+        return resp
+
+    app = web.Application()
+    app.router.add_delete("/storage/user/file", delete_handler)
+
+    srv = await aiohttp_server(app)
+
+    progress = mock.Mock()
+    async with make_client(srv.make_url("/")) as client:
+        await client.storage.rm(URL("storage:file"), progress=progress)
+
+    progress.delete.assert_called_with(
+        StorageProgressDelete(uri=URL("storage://default/user/file"), is_dir=False,)
+    )
+
+
 async def test_storage_rm_directory(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
-    async def get_handler(request: web.Request) -> web.Response:
-        assert request.path == "/storage/user/folder"
-        assert request.query == {"op": "GETFILESTATUS"}
-        return web.json_response(
-            {
-                "FileStatus": {
-                    "path": "/user/folder",
-                    "type": "DIRECTORY",
-                    "length": 1234,
-                    "modificationTime": 3456,
-                    "permission": "read",
-                }
-            }
-        )
-
     async def delete_handler(request: web.Request) -> web.Response:
         assert request.path == "/storage/user/folder"
-        assert request.query == {"op": "DELETE"}
-        return web.Response(status=204)
+        assert request.query == {"op": "DELETE", "recursive": "false"}
+        return web.json_response(
+            {"error": "Target is a directory", "errno": "EISDIR"},
+            status=web.HTTPBadRequest.status_code,
+        )
 
     app = web.Application()
-    app.router.add_get("/storage/user/folder", get_handler)
     app.router.add_delete("/storage/user/folder", delete_handler)
 
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        with pytest.raises(IsADirectoryError, match="Is a directory") as cm:
+        with pytest.raises(IsADirectoryError, match="Target is a directory") as cm:
             await client.storage.rm(URL("storage:folder"))
         assert cm.value.errno == errno.EISDIR
 
@@ -467,10 +479,20 @@ async def test_storage_rm_directory(
 async def test_storage_rm_recursive(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
-    async def delete_handler(request: web.Request) -> web.Response:
+    remove_listing = {
+        "path": "/user/folder",
+        "is_dir": True,
+    }
+
+    async def delete_handler(request: web.Request) -> web.StreamResponse:
         assert request.path == "/storage/user/folder"
-        assert request.query == {"op": "DELETE"}
-        return web.Response(status=204)
+        assert request.query == {"op": "DELETE", "recursive": "true"}
+        assert request.headers["Accept"] == "application/x-ndjson"
+        resp = web.StreamResponse()
+        resp.headers["Content-Type"] = "application/x-ndjson"
+        await resp.prepare(request)
+        await resp.write(json.dumps(remove_listing).encode() + b"\n")
+        return resp
 
     app = web.Application()
     app.router.add_delete("/storage/user/folder", delete_handler)
@@ -666,6 +688,7 @@ async def test_storage_stats(
             size=1234,
             modification_time=3456,
             permission=Action.READ,
+            uri=URL("storage://default/user/folder"),
         )
 
 
@@ -684,7 +707,7 @@ async def test_storage_open(
             return web.json_response(
                 {
                     "FileStatus": {
-                        "path": "/user/file",
+                        "path": "file",
                         "type": "FILE",
                         "length": 5,
                         "modificationTime": 3456,
