@@ -36,11 +36,6 @@ def test_e2e_storage(data: Tuple[Path, str], tmp_path: Path, helper: Helper) ->
         "foo", "folder", checksum, str(tmp_path), "bar"
     )
 
-    # Download into deeper local dir and confirm checksum
-    localdir = tmp_path / "baz"
-    localdir.mkdir()
-    helper.check_file_on_storage_checksum("foo", "folder", checksum, localdir, "foo")
-
     # Rename file on the storage
     helper.check_rename_file_on_storage("foo", "folder", "bar", "folder")
     helper.check_file_exists_on_storage("bar", "folder", FILE_SIZE_B)
@@ -100,8 +95,8 @@ def test_e2e_mkdir(helper: Helper) -> None:
 
 
 @pytest.mark.e2e
-def test_copy_local_file_to_platform_directory(helper: Helper, data: _Data) -> None:
-    srcfile, checksum = data
+def test_copy_local_file_to_platform_directory(helper: Helper, data2: _Data) -> None:
+    srcfile, checksum = data2
     file_name = str(PurePath(srcfile).name)
 
     helper.mkdir("folder", parents=True)
@@ -109,14 +104,14 @@ def test_copy_local_file_to_platform_directory(helper: Helper, data: _Data) -> N
     helper.run_cli(["storage", "cp", srcfile, helper.tmpstorage + "/folder"])
 
     # Ensure file is there
-    helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B)
+    helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B // 3)
 
 
 @pytest.mark.e2e
 def test_copy_local_file_to_platform_directory_explicit(
-    helper: Helper, data: _Data
+    helper: Helper, data2: _Data
 ) -> None:
-    srcfile, checksum = data
+    srcfile, checksum = data2
     file_name = str(PurePath(srcfile).name)
 
     helper.mkdir("folder", parents=True)
@@ -124,7 +119,7 @@ def test_copy_local_file_to_platform_directory_explicit(
     helper.run_cli(["storage", "cp", "-t", helper.tmpstorage + "/folder", srcfile])
 
     # Ensure file is there
-    helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B)
+    helper.check_file_exists_on_storage(file_name, "folder", FILE_SIZE_B // 3)
 
 
 @pytest.mark.e2e
@@ -146,10 +141,10 @@ def test_copy_local_single_file_to_platform_file(helper: Helper, data: _Data) ->
 
 @pytest.mark.e2e
 def test_copy_local_single_file_to_platform_file_explicit(
-    helper: Helper, data: _Data
+    helper: Helper, data2: _Data
 ) -> None:
     # case when copy happens with rename to 'different_name.txt'
-    srcfile, checksum = data
+    srcfile, checksum = data2
     file_name = str(PurePath(srcfile).name)
 
     helper.mkdir("folder", parents=True)
@@ -165,7 +160,9 @@ def test_copy_local_single_file_to_platform_file_explicit(
     )
 
     # Ensure file is there
-    helper.check_file_exists_on_storage("different_name.txt", "folder", FILE_SIZE_B)
+    helper.check_file_exists_on_storage(
+        "different_name.txt", "folder", FILE_SIZE_B // 3
+    )
     helper.check_file_absent_on_storage(file_name, "folder")
 
 
@@ -256,12 +253,12 @@ def test_e2e_copy_no_target_directory_extra_operand(
 
 @pytest.mark.e2e
 def test_copy_and_remove_multiple_files(
-    helper: Helper, data: _Data, data2: _Data, tmp_path: Path
+    helper: Helper, data2: _Data, data3: _Data, tmp_path: Path
 ) -> None:
     helper.mkdir("")
     # case when copy happens with rename to 'different_name.txt'
-    srcfile, checksum = data
-    srcfile2, checksum2 = data2
+    srcfile, checksum = data2
+    srcfile2, checksum2 = data3
     srcname = os.path.basename(srcfile)
     srcname2 = os.path.basename(srcfile2)
 
@@ -270,8 +267,8 @@ def test_copy_and_remove_multiple_files(
     assert captured.out == ""
 
     # Confirm files has been uploaded
-    helper.check_file_exists_on_storage(srcname, "", FILE_SIZE_B)
-    helper.check_file_exists_on_storage(srcname2, "", FILE_SIZE_B // 3)
+    helper.check_file_exists_on_storage(srcname, "", FILE_SIZE_B // 3)
+    helper.check_file_exists_on_storage(srcname2, "", FILE_SIZE_B // 5)
 
     # Download into local directory and confirm checksum
     targetdir = tmp_path / "bar"
@@ -319,7 +316,7 @@ def test_e2e_copy_recursive_to_platform(
     assert not captured.out
 
     helper.check_file_exists_on_storage(
-        target_file_name, f"nested/directory/for/test", FILE_SIZE_B
+        target_file_name, f"nested/directory/for/test", FILE_SIZE_B // 3
     )
 
     # Download into local directory and confirm checksum
